//debug mode (sassert) will be turned off when defined
//#define NDEBUG

#include <DSOL.hpp>


/*
* see http://www.coranac.com/tonc/text/affine.htm for more information about affine matrices.
*/


//generated with grit
#include "vuurrobin.h"
const uint                  vuurrobinOAMIndex      = 0;
const SpriteSize            vuurrobinSize          = SpriteSize_64x64;
const SpriteColorFormat     vuurrobinColorFormat   = SpriteColorFormat_256Color;



int main()
{
	using namespace DSOL;
	using namespace Sprites;

	init2d();

    //========================
    //first constructor
	//========================
	Sprite vuurrobin(vuurrobinOAMIndex,
                vuurrobinTiles, vuurrobinTilesLen, vuurrobinPal, vuurrobinPalLen,//generated with grit
                vuurrobinSize, vuurrobinColorFormat,
                true,               //main oam (defaults to top screen)
                true);             //affine matrix (rotating, scaling, ect)

    vuurrobin.setPosition(50, 30).setDoubleSizeOnRotation(true).update();//chaining commands, so we don't have to type as much (can't chain after update)

    //if something went wrong with getting an affine matrix (like no memory or matrices left),
    //the affine matrix pointer will be NULL
    if(vuurrobin.getAffineMatrixPointer() != NULL)
    {
        //the sprite has an affine matrix
        vuurrobin.getAffineMatrixPointer()->
            setAffineTransformation((1<<8), (1<<8), 0, (1<<8)).update();//chaining commands, so we don't have to type as much
    }


    //========================
    //second constructor
    //========================

    //TODO: add some checks to see if everything is correctly allocated

    //allocating vram and writing to it.
    u16* gfx = oamAllocateGfx(&oamSub, vuurrobinSize, vuurrobinColorFormat);
    copyGfxData(vuurrobinTiles, gfx, vuurrobinTilesLen);

    //reserving a palette and writing to it.
    uint vuurrobinPaletteIndex = Palette::reserveEmptyPalette(false, false);//sub oam and 256 colors
    Palette::writeToPalette(false, false, vuurrobinPaletteIndex, vuurrobinPal, vuurrobinPalLen);

    //create a AffineMatrix object and set the angle and scale
    AffineMatrix rotScale(AffineMatrix::reserveEmptyAffineMatrix(), false);
    rotScale.setAffineTransformation((1<<7), -(1<<9), (3<<6), (1<<9)).update();


    //creating the sprite
    Sprite vuurrobin2(vuurrobinOAMIndex, vuurrobinSize, vuurrobinColorFormat, gfx, vuurrobinPaletteIndex,
                    false, &rotScale);//sub oam and affine matrix (rotating, scaling, ect)
    vuurrobin2.setPosition(50, 30).setDoubleSizeOnRotation(true).update();//chaining commands, so we don't have to type as much



    //update everything and loop forever
    sleepForever();

    //freeing stuff we have used for the second constructor
    AffineMatrix::freeAffineMatrix(rotScale.getIndex());
    Palette::freePalette(false, false, vuurrobinPaletteIndex); //sub oam and 256 colors
    oamFreeGfx(&oamMain, gfx);

	return 0;
}
