#ifndef EXPLOSIONSPRITE_HPP_INC
#define EXPLOSIONSPRITE_HPP_INC


//generated with grit
#include "explosion.h"
const SpriteSize            explosionSize          = SpriteSize_64x64;
const SpriteColorFormat     explosionColorFormat   = SpriteColorFormat_256Color;
//the explosion image has 7 frames
static const uint MAX_FRAMES = 7;


/*
* this class will take care of creating and animating an explosion sprite using the second Sprite constructor.
* all the explosion frames will be loaded into vram memory at the same time and every sprite will use those frames.
* this will use more vram in the beginning (7 frames, so 7 times more vram), but vram usage will stay consistent,
* no matter how many explosion sprites you have. every sprite will also use the same palette.
* plus, it will actually save vram when you have more explosion sprites then frames (so, more then 7 sprites)
*/
class ExplosionSprite : public DSOL::Sprites::Sprite
{
    private:
        //an array that holds the pointers to the frames in vram
        static u16* frames[MAX_FRAMES];

        //the palette index that the explosion uses
        static uint explosionPaletteIndex;

        //a boolean value to see if the class data has already been initialized
        static bool isInitialized;


        //inits the data and returns a pointer to the first frame
        //needed to pass the correct data to sprites constructor
        static inline u16* getInitFrame()
        {
            initializeData();

            return frames[0];
        }


    public:
        //a class function to allocate and initialize the allocated vram and palette
        static void initializeData();

        //a class function to release the allocated vram and palette
        static void releaseData();


    private:
        //the number of vblanks the sprite should wait between swiching frames
        uint vblankPerFrame;
        //the number of vblanks that has passed this current frame
        uint frameVBlanksPassed;
        //the number of vblanks the sprite should wait between explosions
        uint vblankPerExplosion;
        //the number of vblanks that has passed between explosions
        //if its equal to VBLANK_PER_EXPLOSION, then its displaying an explosion
        uint explosionVBlankPassed;


        //overwriting Sprite::setFrame()
        ExplosionSprite& setFrame(const uint newFrame = 0);


    public:
        //the constructor for ExplosionSprite
        ExplosionSprite(const uint index);


        //the update function that will handle the animation
        //overwrites Sprite::update()
        void update();


};//end class ExplosionSprite




#endif // EXPLOSIONSPRITE_HPP_INC
