//debug mode (sassert) will be turned off when defined
//#define NDEBUG

#include <DSOL.hpp>

//also includes stuff from grit
#include "ExplosionSprite.hpp"


int main()
{
	using namespace DSOL;
	using namespace Sprites;

	init2d();

    //========================
    //first constructor
	//========================
	Sprite explosion(0,//the index
                explosionTiles, explosionTilesLen, explosionPal, explosionPalLen,//generated with grit
                explosionSize, explosionColorFormat,
                true,               //main oam (defaults to top screen)
                false);             //no affine matrix (rotating, scaling, ect)

    explosion.setPosition(50, 50).update();//chaining commands, so we don't have to type as much (can't chain after update)


    //the maximum number of frames the animation has
    const uint MAXFRAMES = explosion.getMaxNumberOfFrames();

    //the number of vblanks the sprite should wait between swiching frames
    const uint VBLANK_PER_FRAME = 7;
    //the number of vblanks that has passed this current frame
    uint frameVBlanksPassed = 0;
    //the number of vblanks the sprite should wait between explosions
    const uint VBLANK_PER_EXPLOSION = 30;
    //the number of vblanks that has passed between explosions
    //if its equal to VBLANK_PER_EXPLOSION, then its displaying an explosion
    uint explosionVBlankPassed = VBLANK_PER_EXPLOSION;


    //========================
    //second constructor
    //========================
    //because the ExplosionSprite class takes care of everything,
    //we only have to create objects of it and call the update function once before vblank.

    //the number of explosions there are on the sub engine
    const uint MAX_EXPLOSIONS = 10;

    LiteVector<ExplosionSprite> explosions(MAX_EXPLOSIONS);
    for(uint i = 0; i < MAX_EXPLOSIONS; ++i)
    {
        explosions.push(ExplosionSprite(i));
    }


    //an endless loop that will update the explosions
    while(true)
    {
        //===========================
        //updating first constructor
        //===========================
        if(explosionVBlankPassed == VBLANK_PER_EXPLOSION)
        {
            //the sprite is displaying an explosion
            frameVBlanksPassed++;
            if(frameVBlanksPassed == VBLANK_PER_FRAME)
            {
                //time for the next frame, or wait if the explosion is over
                register uint newFrame = (explosion.getCurrentFrame() + 1);
                if(newFrame == MAXFRAMES)
                {
                    //the explosion is over, so we make the sprite invisible,
                    //and wait a number of vblanks
                    explosion.setVisible(false).update();
                    explosionVBlankPassed = 0;
                }
                else
                {
                    //set the next frame
                    explosion.setFrame(newFrame).update();
                }

                frameVBlanksPassed = 0;
            }
        }
        else
        {
            //the sprite is waiting between explosions
            explosionVBlankPassed++;
            if(explosionVBlankPassed == VBLANK_PER_EXPLOSION)
            {
                //waiting is over, so set the first frame and make the sprite visible
                explosion.setFrame(0).setVisible(true).update();
            }
        }


        //============================
        //updating second constructor
        //============================
        //because the ExplosionSprite class takes care of everything,
        //we only have to call the update function
        for(uint i = 0; i < MAX_EXPLOSIONS; ++i)
        {
            explosions[i].update();
        }


        waitForVBlank();
    }

    ExplosionSprite::releaseData();

	return 0;
}
