#ifndef RANDOM_HPP_INC
#define RANDOM_HPP_INC

/** @file Random.hpp
    @brief this file has some functions for creating random values.
*/


namespace DSOL
{

/**
    @brief sets the rand seed using the current time.
*/
static inline void seedRandDefault()
{
    srand( time(NULL) );
}


/**
    @brief returns a random number between min and max.

    @param min the lower bound of the random number, inclusive.
    @param max the upper bound of the random number, inclusive.

    @return a random number between min and max.
*/
static inline const int randInt(const int min, const int max)
{
    return (min + (rand() % (max - min + 1)));
}


/**
    @brief returns a random boolean value.

    @return randomly true or false.
*/
static inline const bool randBool()
{
    return ((rand() % 2) == 0);
}


}//end namespace DSOL

#endif // RANDOM_HPP_INC
