//including libnds
#include <nds.h>

#include "Palette.hpp"


namespace DSOL
{

namespace Sprites
{

BooleanArray Palette::mainPalette(NUMBER_OF_PALETTES * 2);//2 palettes per array
BooleanArray Palette::subPalette(NUMBER_OF_PALETTES * 2);//2 palettes per array



/*static*/ uint Palette::reserveEmptyPalette(const bool mainOAM, const bool palette16c)
{
    register BooleanArray& tempPalette = (mainOAM ? mainPalette : subPalette);

    //calculates which part of the palette we need,
    //the lower part is for 16 color palettes and the upper part for 256 color palettes
    register const uint COLORINDEX = (palette16c ? 0: NUMBER_OF_PALETTES);

    for(uint i = 0; i < NUMBER_OF_PALETTES; i++)
    {
        if(tempPalette.get(i + COLORINDEX) == false)
        {
            tempPalette.set(i + COLORINDEX, true);
            return i;
        }
    }

    sassert(false, "error: no palettes left");

    //we have to return something
    return NUMBER_OF_PALETTES;
}



/*static*/ void Palette::freePalette(const bool mainOAM, const bool palette16c, const uint paletteIndex)
{
    sassert(paletteIndex < NUMBER_OF_PALETTES, "freePalette() paletteIndex is out of bounds");

    register BooleanArray& tempPalette = (mainOAM ? mainPalette : subPalette);

    //calculates which part of the palette we need,
    //the lower part is for 16 color palettes and the upper part for 256 color palettes
    register const uint COLORINDEX = (palette16c ? 0: NUMBER_OF_PALETTES);

    tempPalette.set(paletteIndex + COLORINDEX, false);
}


/*static*/ void Palette::writeToPalette16c(const bool mainOAM, const uint paletteIndex, const u16* palettePointer,
                                const uint paletteLength)
{
    sassert(paletteLength <= 16*sizeof(u16), "error: palette length is bigger than allowed length size");
    if(mainOAM)
    {
        dmaCopyHalfWords(DMA_PRIORITY, palettePointer, (SPRITE_PALETTE + (paletteIndex * PALETTE_INDEX_MULTIPLIER)), paletteLength);
    }
    else
    {
        dmaCopyHalfWords(DMA_PRIORITY, palettePointer, (SPRITE_PALETTE_SUB + (paletteIndex * PALETTE_INDEX_MULTIPLIER)), paletteLength);
    }
}



/*static*/ void Palette::writeToPalette256c(const bool mainOAM, const uint paletteIndex, const u16* palettePointer,
                                const uint paletteLength)
{
    sassert(paletteLength <= 256*sizeof(u16), "error: palette length is bigger than allowed length size");
    if(mainOAM)
    {
        vramSetBankG(VRAM_G_LCD);//enabling vram G for direct writing acces
        dmaCopyHalfWords(DMA_PRIORITY, palettePointer, VRAM_G_EXT_PALETTE[paletteIndex], paletteLength);
        vramSetBankG(VRAM_G_SPRITE_EXT_PALETTE);
    }
    else
    {
        vramSetBankI(VRAM_I_LCD);//enabling vram I for direct writing acces
        dmaCopyHalfWords(DMA_PRIORITY, palettePointer, VRAM_I_EXT_PALETTE[paletteIndex], paletteLength);
        vramSetBankI(VRAM_I_SUB_SPRITE_EXT_PALETTE);
    }
}


/*static*/ void Palette::copyPalette16c(const bool paletteAmainOAM, const uint paletteAIndex, const bool paletteBmainOAM,
                                    const uint paletteBIndex)
{
    sassert(paletteAIndex < NUMBER_OF_PALETTES, "error: copyPalette16c() palette A index is out of bounds, must be 0-NUMBER_OF_PALETTES");
    sassert(paletteBIndex < NUMBER_OF_PALETTES, "error: copyPalette16c() palette B index is out of bounds, must be 0-NUMBER_OF_PALETTES");

    register u16* copyFrom  = ((paletteAmainOAM? SPRITE_PALETTE: SPRITE_PALETTE_SUB) + (paletteAIndex * PALETTE_INDEX_MULTIPLIER));
    register u16* copyTo    = ((paletteBmainOAM? SPRITE_PALETTE: SPRITE_PALETTE_SUB) + (paletteBIndex * PALETTE_INDEX_MULTIPLIER));

    dmaCopyHalfWords(DMA_PRIORITY, copyFrom, copyTo, 16*sizeof(u16));
}





/*static*/ void Palette::copyPalette256c(const bool paletteAmainOAM, const uint paletteAIndex, const bool paletteBmainOAM,
                            const uint paletteBIndex)
{
    sassert(paletteAIndex < NUMBER_OF_PALETTES, "error: copyPalette256c() palette A index is out of bounds, must be 0-NUMBER_OF_PALETTES");
    sassert(paletteBIndex < NUMBER_OF_PALETTES, "error: copyPalette256c() palette B index is out of bounds, must be 0-NUMBER_OF_PALETTES");


    if(paletteAmainOAM || paletteBmainOAM)
        vramSetBankG(VRAM_G_LCD);//enabling vram G for direct writing acces

    if(!(paletteAmainOAM && paletteBmainOAM))
        vramSetBankI(VRAM_I_LCD);//enabling vram I for direct writing acces

    register u16* copyFrom  = (paletteAmainOAM? VRAM_G_EXT_PALETTE[paletteAIndex][0]: VRAM_I_EXT_PALETTE[paletteAIndex][0]);
    register u16* copyTo    = (paletteBmainOAM? VRAM_G_EXT_PALETTE[paletteBIndex][0]: VRAM_I_EXT_PALETTE[paletteBIndex][0]);

    dmaCopyHalfWords(DMA_PRIORITY, copyFrom, copyTo, 16*sizeof(u16));


    if(paletteAmainOAM || paletteBmainOAM)
        vramSetBankG(VRAM_G_SPRITE_EXT_PALETTE);

    if(!(paletteAmainOAM && paletteBmainOAM))
        vramSetBankI(VRAM_I_SUB_SPRITE_EXT_PALETTE);
}



}//end namespace Sprites

}//end namespace DSOL
